package sd.graphalgorithms;

import Jcg.polyhedron.Halfedge;
import Jcg.polyhedron.Polyhedron_3;
import Jcg.polyhedron.Vertex;
import Jcg.util.DLinkedList;
import Jcg.util.DListNode;

/**
 * A class providing implementation of algorithms involving an edge path on a triangle mesh <p>
 * Remark: the edges are stored in a Double Linked List (storing the halfedges)
 * 
 * @author Luca Castelli Aleardi, Ecole Polytechnique
 * @version april 2017
 */
public class EdgePath {

    /**  edges defining a path of edges in a triangle mesh */
    public DLinkedList<Halfedge> edges;
    
    public EdgePath() {
    	this.edges=new DLinkedList<Halfedge>();
    }

    /**
     * Check whether the input path is simple
     * <p>
     * Remark: the vertices are assumed to have an index, between 0..n-1
     * 
     * @return true if the input path is simple (no vertex appears twice or more times)
     */		   
    public boolean isSimplePath(Polyhedron_3 mesh){
    	if(edges==null)
    		throw new Error("Error: edge path not defined");
    	
    	boolean result=true;
    	
    	System.out.print(" - Checking edge path...");
    	int n=mesh.sizeOfVertices(); // number of vertices in the mesh
    	
    	short[] visited=new short[n]; // the set of vertices already visited
    	DListNode<Halfedge> pNode=this.edges.getFirst();
    	
    	// set the multiplicity of the source node (source of the first edge)
    	Vertex lastVertex=this.edges.getLast().getElement().getOpposite().getVertex();
    	visited[lastVertex.index]=1;
    	
    	int count=0;
    	while(pNode.getNext()!=null) {
    		Vertex v=pNode.getElement().getVertex();
    		visited[v.index]++; // increment the counter (count the number of occurrences of 'v')
    		if(visited[v.index]>1) {
    			System.out.println("Error: the (edge) path is not simple: vertex "+v.index+" appears twice");
    			result=false;
    		}
    		pNode=pNode.getNext(); // move to the next cell in the list
    		count++;
    	}
    	
    	System.out.println("ok the edge path is simple ("+count+" edges)");
    	return result;
    }

}
