package sd.graphalgorithms;

import Jcg.polyhedron.Halfedge;
import Jcg.util.DListNode;

/**
 * Data structure for dealing with 'free vertices': it stores the set of free vertices
 * on the cut-border in a queue, using an array-based implementation.
 * 
 * (based on code by Mark Allen Weiss)
 */
public class FreeVertices {

    private Object[] array;
    private int currentSize=0;
    private int first=0;
    private int last=0;
    public final int maxSize;
   // public boolean[] isExisting;

    /**
     * Initialize an empty queue, storing at most 'n' elements
     */
    public FreeVertices(int n) {
    	this.maxSize=n;
    	this.array=new Object[n];
    	//this.isExisting=new boolean[n];
    }
    
    /**
     * Check whether the queue is empty (no more free vertices)
     */
    public boolean isEmpty( ) {
        return currentSize == 0;
    }

    /**
     * Return the number of nodes in the queue
     */
    public int size() {
        return currentSize;
    }

    /**
     * Return the first element in the queue
     */
    public Object getFirst( ) {
        if(this.isEmpty()==true)
            return null;
        return array[first];
    }

    /**
     * Return and remove the last node in the queue
     */
    public DListNode<Halfedge> poll() {
        if(isEmpty()==true)
            return null;
        
        currentSize--;

        DListNode<Halfedge> firstElement=(DListNode<Halfedge>)array[first];
        array[first]=null;
        this.first=increment(first);
        
        //if(firstElement!=null)
        //this.isExisting[firstElement.getElement().getVertex().index]=false;
        
        return firstElement;
    }

    /**
     * Append a new node at the end of the queue
     */
    public void addLast(DListNode<Halfedge> node) {
    	//if(this.isExisting[node.getElement().getVertex().index]==true)
    	//	return;
    	
    	if(currentSize>this.maxSize)
    		throw new Error("Error: the current of the queue exceeds the capacity");
    	
        array[last]=node;
        this.last=increment(last);
        currentSize++;
        
        //this.isExisting[node.getElement().getVertex().index]=true;
    }

    /**
     * Internal method to increment with wraparound.
     * @param x any index in theArray's range.
     * @return x+1, or 0, if x is at the end of theArray.
     */
    private int increment(int x ) {
        //if( ++x == array.length )
        //    x = 0;
        return (x+1)%array.length;
    }

}
