package sd.graphalgorithms;

import java.util.*;

import jdg.graph.AdjacencyListGraph;

/**
 * A class representing a hierarchical sequence of graphs: graph G_{i+1} is obtained from G_{i} with a coarsening
 * process based on edge collapses.
 * 
 * @author Luca Castelli Aleardi, Ecole Polytechnique
 * @version august 2017
 */
public class GraphHierarchy {

	private ArrayList<AdjacencyListGraph> sequence; // sequence of coarser graphs
	
	public GraphHierarchy() {
		this.sequence=new ArrayList<AdjacencyListGraph>();
	}
	
	/**
	 * Return the i-th graph in the graph hierarchy <p>
	 * Remark: <p>
	 * -) the graph at level i=0 is the original (largest graph) <p>
	 * -) the last graph is the graph at the bottom of the hierarchy (the smallest one)
	 * 
	 */
	public AdjacencyListGraph getGraph(int i) {
		if(i<0 || i>=this.sequence.size())
			throw new Error("Error: wrong index of graph i="+i);
		
		return this.sequence.get(i);
	}

	/**
	 * Add a new graph to the graph hierarchy
	 */
	public void addGraph(AdjacencyListGraph g) {
		this.sequence.add(g);
	}

	/**
	 * Return the number of graphs in the hierarchy (including the first input graph)
	 */
	public int size() {
		return this.sequence.size();
	}

}
