package sd.layout;

import Jcg.geometry.Point_3;
import Jcg.geometry.Vector_3;
import jdg.graph.AdjacencyListGraph;
import jdg.graph.Node;
import sd.util.SphericalGeometry;

/**
 * This class implement the standard (slow) computation of repulsive forces (between all vertices)
 * 
 * @author Luca Castelli Aleardi, Ecole Polytechnique
 * @version april 2017
 */
public class ExactForceComputation extends ForceComputation {
	
	public ExactForceComputation(ForceModel force) {
		this.force=force;
	}
	
	/**
	 * Compute the displacement of vertex 'u': slow method
	 * 
	 * @param u  the vertex to which repulsive forces are applied
	 * @return 'displacement' a 3d vector storing the displacement of vertex 'u'
	 */	
	public Vector_3 computeRepulsiveForce(AdjacencyListGraph g, Node u, double tol) {
		Vector_3 ex=new Vector_3(1.,0.,0.);
		Vector_3 ey=new Vector_3(0.,1.,0.);
		Vector_3 ez=new Vector_3(0.,0.,1.);
		Vector_3 displacement=new Vector_3(0., 0., 0.);
		Vector_3 delta;
		Point_3 p=u.getPoint();
		Vector_3[]tangentialVectors=SphericalGeometry.getTangentialVector(p,0.01);
		Vector_3 v1=tangentialVectors[0];
		Vector_3 v2=tangentialVectors[1];
		double distance=0.;
		for(Node v: g.vertices){
			if(v!=null && u!=v && v.degree()>=0){
				distance=SphericalGeometry.geodesicDistance(p,v.getPoint());
				delta=new Vector_3(p,v.getPoint());
				delta=new Vector_3(v1.innerProduct(delta).doubleValue()*v1.innerProduct(ex).doubleValue()+v2.innerProduct(delta).doubleValue()*v2.innerProduct(ex).doubleValue(),delta.innerProduct(v1).doubleValue()*v1.innerProduct(ey).doubleValue()+delta.innerProduct(v2).doubleValue()*ey.innerProduct(v2).doubleValue(),delta.innerProduct(v1).doubleValue()*v1.innerProduct(ez).doubleValue()+delta.innerProduct(v2).doubleValue()*v2.innerProduct(ez).doubleValue());
				double norm=Math.sqrt(delta.squaredLength().doubleValue());
				if(norm>tol){
					delta=delta.multiplyByScalar(-1.0/norm);
				}
				else{
					delta=new Vector_3(0.,0.,0.);
				}
				displacement=displacement.sum(delta.multiplyByScalar(this.force.repulsiveForce(distance)));
			}
		}
		return displacement;
	}

}
