package sd.layout;

/**
 * This class define the abstract model of a force
 * 
 * @author Luca Castelli Aleardi, Ecole Polytechnique
 * @version april 2017
 */
public abstract class ForceModel {

	/** count repulsive forces*/	
	public int countRepulsive=0; // count the number of computed repulsive forces (to measure time performances)
	
	/**
	 * Compute the (intensity of the) attractive force between two nodes at a given distance
	 * 
	 * @param distance  distance between two nodes
	 */	
	public abstract double attractiveForce(double distance);
	
	/**
	 * Compute the energy corresponding to the attractive force between two nodes at a given distance
	 * 
	 * @param distance  distance between two nodes
	 */	
	public abstract double attractiveEnergy(double distance);
	
	/**
	 * Compute the (intensity of the) repulsive force between two nodes at a given distance
	 * 
	 * @param distance  distance between two nodes
	 */	
	public abstract double repulsiveForce(double distance);

	/**
	 * Return the weighted version of the repulsive force acting on vertex 'u'
	 * 
	 * @param distance  distance between two nodes
	 * @param weight  the weight of vertex 'u'
	 */	
	public double repulsiveForce(double distance, double weight) {
		return this.repulsiveForce(distance)*weight;
	}

	/**
	 * Compute the energy corresponding to the repulsive force between two nodes at a given distance
	 * 
	 * @param distance  distance between two nodes
	 */	
	public abstract double repulsiveEnergy(double distance);
	
	/**
	 * Compute the (intensity of the) gravity force on a node (directed toward the gravity center)
	 * 
	 * @param distance  distance a node and the gravity center
	 */	
	public abstract double gravityForce(double distance);

}
