package sd.layout;

/**
 * An interface defining all methods supported by 'force-directed' layout algorithms
 * 
 * @author Luca Castelli Aleardi, Ecole Polytechnique
 * @version august 2017
 */
public interface SpringEmbedder {
	/** Get the current value of the temperature */
	public double getTemperature();

	/** Get the value of the minimal temperature */
	public double getMinimalTemperature();

	/** Get the value of the cooling constant */
	public double getCoolingConstant();

	/** Set the initial value of the temperature */
	public void setInitialTemperature(double t);

	/** Set the initial value of the cooling constant */
	public void setCoolingConstant(double t);

	/** Set the minimal value of the temperature */
	public void setMinimalTemperature(double t);

	/** Set the value of the parameter 'C', the relative strength of the repulsive forces (as used in Hu 2006) */
	public void setC(double c);

	/** Get the value of the parameter 'C', the relative strength of the repulsive forces (as used in Hu 2006) */
	public double getC();

	/** Set the value of the parameter 'K', the natural spring length */
	public void setK(double k);

	/** Get the value of the parameter 'K', the natural spring length */
	public double getK();

	/** Set the value of the step length (as described in Hu 2006) */
	public void setStep(double step);

	/** Get the current value of step length */
	public double getStep();

	/** Enable cooling process */	
	public void enableCooling();
	
	/** Disable cooling process */	
	public void disableCooling();

	/** decide to move all vertices all togerther, after computing all mutual forces (as in the original FR91 layout algorithm) */	
	public void moveAllVerticesAtOnce();

	/** decide to move only one vertex at a time, computing forces and displacement only for that vertex (as in Walshaw 2003) */	
	public void moveOnlyOneVertex();

	/** use vertex weights for computing repulsive forces (as in Walshaw 2003) */	
	public void useVertexWeights();

	/** do not use vertex weights for computing repulsive forces (as in FR91) */	
	public void noVertexWeights();

}
