package sd.viewer;
import tc.TC;

/**
 * A class for reading input parameters from config file (using Processing)
 *
 * @author Luca Castelli Aleardi (Ecole Polytechnique, 2017)
 */
public class ConfigSphericalDrawing {
	
	static private double tolerance; // numeric tolerance (for convergence)
	static double threshold; // numeric threshold (for convergence of iterative spherical Tutte methods)
	static private int maxIterations; // maximum number of iterations to perform
	static private int vertexSeed; // vertex seed for the computation of shortests paths
	static int southFaceIndex, northFaceIndex; // indices of the south and north pole faces
	static private int minDistance; // parameters for the computation of the separating cycle
	static private String linearSolver; // choice of the solver for lienar systems (Jama, MTJ, ParallelColt)
	static int FPPsteps=0, FPPavoid=0; // input parameters for the computation of the canonical labeling
	static boolean useEqualAngles=false; // say whether to use 'equal' angles of 'adapted' angles (for the prism)

	public static void readInputParameters(String configLayout) {
		readLayout(configLayout);
	}
	private static void readLayout(String configFileName) {
		
		System.out.println("Reading input parameters from: "+configFileName);
		TC.lectureDansFichier(configFileName);
		
		while(TC.finEntree()==false) {
			String ligne=TC.lireLigne();
			String[] tab=TC.motsDeChaine(ligne);
			if(tab.length>0) {
				System.out.println("\t"+tab[0]+"\t   "+tab[1]);
				readLayoutParameter(tab[0], tab[1]);
			}
		}
		TC.lectureEntreeStandard();
	}
	
	private static void readLayoutParameter(String parameterType, String value) {
		if(parameterType.equals("tolerance"))
			tolerance=Double.parseDouble(value);
		else if(parameterType.equals("threshold"))
			threshold=Double.parseDouble(value);
		else if(parameterType.equals("vertexSeed"))
			vertexSeed=Integer.parseInt(value);
		else if(parameterType.equals("minDistance"))
			minDistance=Integer.parseInt(value);
		else if(parameterType.equals("linearSolver"))
			linearSolver=value;
		else if(parameterType.equals("maxIterations"))
			maxIterations=Integer.parseInt(value);
		else if(parameterType.equals("southFace"))
			southFaceIndex=Integer.parseInt(value);
		else if(parameterType.equals("northFace"))
			northFaceIndex=Integer.parseInt(value);
		else if(parameterType.equals("FPPsteps"))
			FPPsteps=Integer.parseInt(value);
		else if(parameterType.equals("FPPavoid"))
			FPPavoid=Integer.parseInt(value);
		else if(parameterType.equals("equalAngles")) {
			if(value.equals("true"))
				useEqualAngles=true;
			else if(value.equals("false"))
				useEqualAngles=false;
			else
				System.out.println("Warning: input parameter not defined: "+value);
		}

		else
			throw new Error("Error: wrong input Layout parameter ("+parameterType+")");
	}
		
	public static void setParameters() {
		System.out.print("Setting input parameters...");
		MeshRepresentation.tolerance=ConfigSphericalDrawing.tolerance;
		MeshRepresentation.threshold=ConfigSphericalDrawing.threshold;
		MeshRepresentation.vertexSeed=ConfigSphericalDrawing.vertexSeed;
		MeshRepresentation.minDistance=ConfigSphericalDrawing.minDistance;
		MeshRepresentation.linearSolver=ConfigSphericalDrawing.linearSolver;
		MeshRepresentation.maxIterations=ConfigSphericalDrawing.maxIterations;
		MeshRepresentation.southFaceIndex=ConfigSphericalDrawing.southFaceIndex;
		MeshRepresentation.northFaceIndex=ConfigSphericalDrawing.northFaceIndex;
		MeshRepresentation.FPPsteps=ConfigSphericalDrawing.FPPsteps;
		MeshRepresentation.FPPavoid=ConfigSphericalDrawing.FPPavoid;
		MeshRepresentation.useEqualAngles=ConfigSphericalDrawing.useEqualAngles;

		System.out.println("done");
	}

}
