package sd.viewer;

import java.awt.Color;
import java.awt.Font;
import java.awt.Label;
import java.awt.Panel;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import sd.SphericalLayoutComputation;
import sd.layout.Layout3D;
import sd.layout.SphericalFR91;
import sd.layout.SphericalLayout;
import sd.layout.SpringEmbedder;

public class Panel3DEuclideanLayout extends Panel implements ActionListener, ItemListener, ChangeListener, ListSelectionListener {


	private static final long serialVersionUID = 1L;
	private static final int buttonHeight=20;
	private static final int hShift=2; // horizontal shift, for placing widgets

	// --- GUI Widgets ---
	JButton buttonOneIteration, buttonAllIterations, buttonIterateUntilConvergence; // load next (previous) graph in the sequencefieldP;
	JCheckBox checkStatistics; // checkbox, for enabling/disabling the computation of statistics
	
	JTextField fieldIterations;
	public JTextField fieldNumericTolerance;
	public JTextField fieldCoarseningThreshold;
	public JTextField fieldRepulsiveStrenght; // relative strenght of repulsive forces
	public JTextField fieldMinimalTemperature;
	JList<String> listLayoutAlgorithm;
	JLabel labelSpringEmbedderParameters;
	// --- end GUI Widgets ---
	
	SphericalLayoutComputation layoutComputation; // main class (graph viewer)

	public Panel3DEuclideanLayout(SphericalLayoutComputation layoutComputation, int posY, int width, int height) {
		this.layoutComputation=layoutComputation;
		
		//this.setPreferredSize(new Dimension(width, height));
		this.setBounds(new Rectangle(0, posY, width, height));
		this.setBackground(Color.gray);
		//this.setLayout(new GridLayout(10, 2));
		this.setLayout(null);
        this.setVisible(true);
		
        int verticalShift1=60, verticalShift2=90, verticalShift3=160, verticalShift4=185; // y-coordinates for placing blocks of widgets
        
        //create the Jlist storing all available layout algorithms
        DefaultListModel<String> listModel = new DefaultListModel<>();
        listModel.addElement("Standard euclidean FR91");
//        listModel.addElement("Multi-level (Walshaw)");
        
        this.listLayoutAlgorithm = new JList<>(listModel); //create the list
        this.listLayoutAlgorithm.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        this.listLayoutAlgorithm.setSelectedIndex(0);
        //JScrollPane scrollList=new JScrollPane(this.listLayoutAlgorithm);
        this.listLayoutAlgorithm.setBounds(10, 0, 240, 30);
        //scrollList.setBounds(200, 0, 20, 120);
        this.listLayoutAlgorithm.addListSelectionListener(this);
        this.add(this.listLayoutAlgorithm);
        //this.add(scrollList);
        
        // create textfields for the number of iterations to perform
        this.fieldIterations = new JTextField("50");
        this.fieldIterations.addActionListener(this);
        this.fieldIterations.setBounds(170, verticalShift1, 80, buttonHeight);
        this.add(this.fieldIterations);
        Label labelMaxIterations=new Label("iterations");
        labelMaxIterations.setBounds(5, verticalShift1, 160, buttonHeight);
        this.add(labelMaxIterations);

        // create textfield for the numeric tolerance (defining the convergence)
        this.fieldNumericTolerance = new JTextField("0.001");
        this.fieldNumericTolerance.addActionListener(this);
        this.fieldNumericTolerance.setToolTipText("Numeric tolerance for the convergence of the iterative method");
        this.fieldNumericTolerance.setBounds(170, verticalShift1+buttonHeight, 80, buttonHeight);
        //this.add(this.fieldNumericTolerance);
        Label labelNumericTolerance=new Label("numeric tolerance");
        labelNumericTolerance.setBounds(5, verticalShift1+buttonHeight, 160, buttonHeight);
        //this.add(labelNumericTolerance);

        // parameters for the spring embedders
        Label labelSpringEmbedders=new Label("Parameters for Spherical FR91");
        labelSpringEmbedders.setFont(new Font("Helvetica", Font.BOLD, 12));
        labelSpringEmbedders.setBounds(2, verticalShift2, 250, buttonHeight);
        this.add(labelSpringEmbedders);
       
        // create textfield for the coarsening threshold (defining the number of levels in the hierarchy of graphs)
        this.fieldCoarseningThreshold = new JTextField("0.1");
        this.fieldCoarseningThreshold.addActionListener(this);
        this.fieldCoarseningThreshold.setToolTipText("A float value in [0,1] definining the number of levels in the hierarchy [only for multi-level layouts]");
        this.fieldCoarseningThreshold.setBounds(170, verticalShift2+buttonHeight, 80, buttonHeight);
        //this.add(this.fieldCoarseningThreshold);
        Label labelCoarseningThreshold=new Label("coarsening threshold");
        labelCoarseningThreshold.setBounds(5, verticalShift2+buttonHeight, 160, buttonHeight);
        //this.add(labelCoarseningThreshold);

        // create textfield for the relative repulsive strenght of repulsive forces
        this.fieldRepulsiveStrenght = new JTextField("0.1");
        this.fieldRepulsiveStrenght.addActionListener(this);
        this.fieldRepulsiveStrenght.setToolTipText("Relative strenght of repulsive forces (a real value)");
        this.fieldRepulsiveStrenght.setBounds(170, verticalShift2+buttonHeight*1, 80, buttonHeight);
        this.add(this.fieldRepulsiveStrenght);
        Label labelRepulsiveStrenght=new Label(" C (repulsive strenght)");
        labelRepulsiveStrenght.setBounds(5, verticalShift2+buttonHeight*1, 160, buttonHeight);
        this.add(labelRepulsiveStrenght);

        // create textfield for the relative repulsive strenght of repulsive forces
        this.fieldMinimalTemperature = new JTextField("0.02");
        this.fieldMinimalTemperature.addActionListener(this);
        this.fieldMinimalTemperature.setToolTipText("Minimal value of the temperature (for simulated annealing)");
        this.fieldMinimalTemperature.setBounds(170, verticalShift2+buttonHeight*2, 80, buttonHeight);
        this.add(this.fieldMinimalTemperature);
        Label labelMinimalTemperature=new Label("minimal temperature");
        labelMinimalTemperature.setBounds(5, verticalShift2+buttonHeight*2, 160, buttonHeight);
        this.add(labelMinimalTemperature);

		// create button for performing one iteration		
		this.buttonOneIteration = new JButton("one iteration");
		this.buttonOneIteration.setToolTipText("perform one iteration");
		this.buttonOneIteration.addActionListener(this);
		this.buttonOneIteration.setBounds(0, verticalShift3, 130, buttonHeight);
		this.add(this.buttonOneIteration);

		// create button for performing all iterations	
		this.buttonAllIterations = new JButton("all iterations");
		this.buttonAllIterations.setToolTipText("perform all iterations");
		this.buttonAllIterations.addActionListener(this);
		this.buttonAllIterations.setBounds(130+hShift, verticalShift3, 125, buttonHeight);
		this.add(this.buttonAllIterations);
		
		// create button for performing iterations until convergence	
		this.buttonIterateUntilConvergence = new JButton("iterate until convergence");
		this.buttonIterateUntilConvergence.setToolTipText("perform iterations until convergence");
		this.buttonIterateUntilConvergence.addActionListener(this);
		this.buttonIterateUntilConvergence.setBounds(2, verticalShift3+buttonHeight, 250, buttonHeight);
		//this.add(this.buttonIterateUntilConvergence);
		
		// add a checkBox for showing statistics
		this.checkStatistics=new JCheckBox("compute statistics (slow)"); // choose to compute statistics during the layout computation
		this.checkStatistics.setSelected(false);
		this.checkStatistics.addItemListener(this);
		this.checkStatistics.setToolTipText("Compute statistics during the layout computation (every 10 iterations): it increases the computation time");
		this.checkStatistics.setBounds(2, verticalShift4, 270, buttonHeight);
		this.add(this.checkStatistics);
		
		// add buttons and components to the panel
		this.labelSpringEmbedderParameters=new JLabel("k=    C=    T=   minT=");
		this.labelSpringEmbedderParameters.setBounds(2, verticalShift4+10, 240, 40);
		this.add(this.labelSpringEmbedderParameters);
	}
	
	public void actionPerformed(ActionEvent e) {		
		this.updateSpringEmbedderParameters(); // update input parameters reading input fields
		this.showCurrentSpringEmbedderParameters(); // show the current layout parameters
		
		//double coarseningThreshold=Double.parseDouble(this.fieldCoarseningThreshold.getText()); // update the value of the 'coarsening threshold'
		//WalshawLayout walshaw=(WalshawLayout)this.layoutComputation.euclideanLayouts[1];
		//walshaw.setThreshold(coarseningThreshold);
		
		if (e.getSource() == buttonOneIteration) {
			this.layoutComputation.meshRendering.showEuclidean3DLayout();
			this.layoutComputation.controlBar.panelSphericalLayout.buttonAllIterations.setEnabled(false);
			this.layoutComputation.controlBar.panelSphericalLayout.buttonOneIteration.setEnabled(false);
			this.layoutComputation.controlBar.panelSphericalLayout.listLayoutAlgorithm.setEnabled(false);
			
			if(this.layoutComputation.selectedEuclideanLayout!=null) {
				
				boolean statistics=this.checkStatistics.isSelected();
				this.layoutComputation.selectedEuclideanLayout.computeOneIteration();
				if(statistics==true) {
					int countCollidingTriangles=SphericalLayout.countTriangleCollisions(this.layoutComputation.input.mesh, this.layoutComputation.input.graph);
					System.out.println("Collisions:\t"+countCollidingTriangles);
				}
			}

			this.layoutComputation.meshRendering.updateScaleFactor();
		}

		if (e.getSource() == buttonAllIterations) {
			this.layoutComputation.meshRendering.showEuclidean3DLayout();
			this.layoutComputation.controlBar.panelSphericalLayout.buttonAllIterations.setEnabled(false);
			this.layoutComputation.controlBar.panelSphericalLayout.buttonOneIteration.setEnabled(false);
			this.layoutComputation.controlBar.panelSphericalLayout.listLayoutAlgorithm.setEnabled(false);
			int nIterations=Integer.parseInt(this.fieldIterations.getText());
			boolean statistics=this.checkStatistics.isSelected();
			
			for(int i=0; i<nIterations; i++){
				this.layoutComputation.selectedEuclideanLayout.computeOneIteration();
				if(statistics==true) {
					int countCollidingTriangles=SphericalLayout.countTriangleCollisions(this.layoutComputation.input.mesh, this.layoutComputation.input.graph);
					System.out.println("Collisions:\t"+countCollidingTriangles);
				}
			}

		}

		if (e.getSource() == buttonIterateUntilConvergence) {
			double tolerance=Double.parseDouble(this.fieldNumericTolerance.getText());
			
			if(this.layoutComputation.selectedSphericalLayout!=null)
				this.layoutComputation.selectedSphericalLayout.computeLayout(tolerance);
			else if(this.layoutComputation.selectedEuclideanLayout!=null)
				this.layoutComputation.selectedEuclideanLayout.computeLayout(tolerance);
			else {
				System.err.println("Warning: wrong layout selection");
				return;
			}
		}

		this.layoutComputation.meshRendering.updateScaleFactor(); // resize the 3d scene
		//this.repaintButtons();
		//this.drawgraph.layoutControlBar.repaintButtons();
		this.repaint();
		this.layoutComputation.repaint();	// also repaint the canvas
	}
	
	/**
	 * Select a layout algorithm
	 */	
	public void showCurrentSpringEmbedderParameters() {
		// layout parameters to show
		double K=0, C=0, T=0, minT=0, theta=0;
		int levels=0;
				
		// reading parameters of euclidean spring embedders
		if(this.layoutComputation.selectedEuclideanLayout==this.layoutComputation.euclideanLayouts[0]) { // original FR91 (euclidean)
			SpringEmbedder layout=(SpringEmbedder)this.layoutComputation.euclideanLayouts[0];
			
			K=round(layout.getK(), 3);
			C=round(layout.getC(), 3);
			T=round(layout.getTemperature(), 3);
			minT=round(layout.getMinimalTemperature(), 3);
			this.labelSpringEmbedderParameters.setText("K="+K+" C="+C+" T="+T+" minT="+minT);
		}
		/*else if(this.layoutComputation.selectedEuclideanLayout==this.layoutComputation.euclideanLayouts[1]) { // multi-level Walshaw 2003 (euclidean)
			MultiLevelSpringEmbedder layout=(MultiLevelSpringEmbedder)this.layoutComputation.euclideanLayouts[1];
			
			C=round(layout.getC(), 3);
			levels=layout.getHierarchySize();
			this.labelSpringEmbedderParameters.setText(" C="+C+" levels="+levels);
		}*/

	}

	/**
	 * Update the parameters of the 'selected' spring embedding algorithm (spherical or euclidean)
	 */	
	public void updateSpringEmbedderParameters() {
		// read input parameters from the interface fields
		double coarseningThreshold=Double.parseDouble(this.fieldCoarseningThreshold.getText()); // for multi-level layouts
		double C=Double.parseDouble(this.fieldRepulsiveStrenght.getText()); // relative strength of repulsive forces
		double minT=Double.parseDouble(this.fieldMinimalTemperature.getText()); // minimal temperature		
		
		// euclidean spring embedders
		if(this.layoutComputation.selectedEuclideanLayout==this.layoutComputation.euclideanLayouts[0]) { // standard FR91 (euclidean)
			SpringEmbedder layout=(SpringEmbedder)this.layoutComputation.selectedEuclideanLayout;
			
			layout.setC(C);
			layout.setMinimalTemperature(minT);
			return;
		}
		/*if(this.layoutComputation.selectedEuclideanLayout==this.layoutComputation.euclideanLayouts[1]) { // multi-level FR91 (euclidean)
			WalshawLayout layout=(WalshawLayout)this.layoutComputation.selectedEuclideanLayout;
			
			layout.setC(C);
			layout.setMinimalTemperature(minT);
			layout.setCoarseningThreshold(coarseningThreshold);
			return;
		}*/

	}


	@Override
	public void itemStateChanged(ItemEvent arg0) {
	}

	public void valueChanged(ListSelectionEvent evt) { 
		 String name=(String)this.listLayoutAlgorithm.getSelectedValue();
		 this.layoutComputation.updateSelectedLayoutMethod(name);
		 this.showCurrentSpringEmbedderParameters();
	}

	public void repaintButtons() {
	}
	
	public void stateChanged(ChangeEvent e) {
	}
	
	/** Return the value after truncation */
	public static double round(double x, int prec) {
		double precision=Math.pow(10, prec);
		return ((int)(x*precision)/((double)precision));
	}

}
