package sd.viewer;
import java.awt.Color;
import java.awt.Label;
import java.awt.Panel;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import Jcg.polyhedron.Vertex;
import sd.util.MeshAlgorithms;

public class PanelViewer extends Panel implements ActionListener, ItemListener, ChangeListener {

	private static final long serialVersionUID = 1L;
	private static final int buttonHeight=20;
	private static final int hShift=2; // horizontal shift, for placing widgets

	// GUI Widgets
	public JButton buttonZoomIn, buttonZoomOut;
	public JButton buttonShow3DMesh, buttonShowSphericalLayout, buttonShowEuclidean3DLayout, buttonGet3DMeshFromGraph;	
	
	SurfaceMeshRendering meshRendering; // renderer of the 3D scene

	public PanelViewer(SurfaceMeshRendering meshRendering, int posY, int width, int height) {
		if(meshRendering==null) {
			System.err.println("Warning: mesh rendering not defined");
			System.exit(0);
		}
		this.meshRendering=meshRendering;
		
		//this.setPreferredSize(new Dimension(width, height));
		this.setBounds(new Rectangle(0, posY, width, height));
		this.setBackground(Color.gray);
        this.setVisible(true);
        this.setLayout(null);
		
        int verticalShift1=0, verticalShift2=30; // y-coordinates for placing blocks of widgets
        
		// create button for performing zoom in		
		this.buttonZoomIn = new JButton("zoom in");
		this.buttonZoomIn.setToolTipText("perform a zoom in of the 3d scene");
		this.buttonZoomIn.addActionListener(this);
		this.buttonZoomIn.setBounds(0, verticalShift1, 110, buttonHeight);
		this.add(this.buttonZoomIn);

		// create button for performing zoom out	
		this.buttonZoomOut = new JButton("zoom out");
		this.buttonZoomOut.setToolTipText("perform a zoom out of the 3d scene");
		this.buttonZoomOut.addActionListener(this);
		this.buttonZoomOut.setBounds(120, verticalShift1, 130, buttonHeight);
		this.add(this.buttonZoomOut);

		// create button for showing the 3D mesh	
		this.buttonShow3DMesh = new JButton("show 3D mesh");
		this.buttonShow3DMesh.setToolTipText("show the 3D mesh (faces are rendered)");
		this.buttonShow3DMesh.addActionListener(this);
		this.buttonShow3DMesh.setBounds(30, verticalShift2, 200, buttonHeight);
		this.add(this.buttonShow3DMesh);

		// create button for showing the spherical layout
		this.buttonShowSphericalLayout = new JButton("show spherical layout");
		this.buttonShowSphericalLayout.setToolTipText("show the spherical layout of the graph (using geodesics)");
		this.buttonShowSphericalLayout.addActionListener(this);
		this.buttonShowSphericalLayout.setBounds(30, verticalShift2+buttonHeight, 200, buttonHeight);
		this.add(this.buttonShowSphericalLayout);

		// create button for showing the euclidean 3d layout	
		this.buttonShowEuclidean3DLayout = new JButton("show euclidean layout");
		this.buttonShowEuclidean3DLayout.setToolTipText("show the euclidean 3D layout of the graph (using straight-line segments)");
		this.buttonShowEuclidean3DLayout.addActionListener(this);
		this.buttonShowEuclidean3DLayout.setBounds(30, verticalShift2+buttonHeight*2, 200, buttonHeight);
		this.add(this.buttonShowEuclidean3DLayout);

		// create button for showing the euclidean 3d layout	
		this.buttonGet3DMeshFromGraph = new JButton("get 3D from graph");
		this.buttonGet3DMeshFromGraph.setToolTipText("set the coordinates of vertices in the 3D from the coordinates of nodes in the graph");
		this.buttonGet3DMeshFromGraph.addActionListener(this);
		this.buttonGet3DMeshFromGraph.setBounds(30, verticalShift2+buttonHeight*3, 200, buttonHeight);
		this.add(this.buttonGet3DMeshFromGraph);

	}
	
	public void actionPerformed(ActionEvent e) {		
		if (e.getSource() == this.buttonZoomIn) {
			this.meshRendering.zoom(1.2);
		}

		if (e.getSource() == this.buttonZoomOut) {
			this.meshRendering.zoom(0.8);
		}

		if (e.getSource() == this.buttonShow3DMesh) {
			this.meshRendering.show3DMesh();
		}

		if (e.getSource() == this.buttonShowSphericalLayout) {
			this.meshRendering.showSphericalLayout();
			
			// export the spherical layout to IPE format
			ExportLayoutToIpe exportToIpe=new ExportLayoutToIpe(this.meshRendering);
			//exportToIpe.export(this.meshRendering.input.graph, null, "outputSpherical.ipe", "spherical layout");
			exportToIpe.export(this.meshRendering.input.graph, null, "output.ipe", "3D layout");
			//this.exportToOFF(true, false, "output.off");
		}

		if (e.getSource() == this.buttonShowEuclidean3DLayout) {
			this.meshRendering.showEuclidean3DLayout();
		}

		if (e.getSource() == this.buttonGet3DMeshFromGraph) {
			for(Vertex v: this.meshRendering.input.mesh.vertices) {
				if(v.index<this.meshRendering.input.graph.sizeVertices()) // do not project the two south/north poles
					v.setPoint(this.meshRendering.input.graph.vertices.get(v.index).getPoint());
			}
			
			this.meshRendering.show3DMesh();
		}

		//this.repaintButtons();
		//this.drawgraph.layoutControlBar.repaintButtons();
		this.repaint();
		this.meshRendering.view.repaint();	// also repaint the canvas
	}

	@Override
	public void itemStateChanged(ItemEvent arg0) {
	}

	public void repaintButtons() {
	}
	
	public void stateChanged(ChangeEvent e) {
	}
	
	/**
	 * Export the mesh/graph to OFF format
	 * 
	 * @param exportSphericalLayout if true, export the spherical layout (vertices are located on the unit sphere); otherwise export the 3d mesh
	 * @param exportWithColors  if true export the 3D mesh, together with colored edges and faces
	 */	
	/*public void exportToOF(boolean exportSphericalLayout, boolean exportWithColors, String outputFile) {
		if(exportSphericalLayout) {
			int n=this.meshRendering.input.graph.sizeVertices(); // number of vertices in the original graph
			
			if(this.meshRendering.input.mesh.sizeOfVertices()==n) { // the mesh is unchanged (this occur for the 'IS' layout)
				MeshAlgorithms.writePolyhedronToOFF(this.meshRendering.input.mesh, this.meshRendering.input.graph, outputFile, false, false);
			}
			else { // the mesh has been modified: the south and north poles have been added (only for 'prism layout')
				// first remove south and north poles: by definition they have indices 'n' and 'n+1' (last added vertices)
				System.out.print("Removing south and north poles...");
				Vertex north=this.input.mesh.vertices.get(n+1);
				Vertex south=this.input.mesh.vertices.get(n);
				this.input.mesh.eraseCenterVertex(north.getHalfedge());
				this.input.mesh.eraseCenterVertex(south.getHalfedge());
				System.out.println("done");
				
				this.input.mesh.isValid(true); // check the combinatorial validity of the 3d mesh, after vertex removals
				
				MeshAlgorithms.writePolyhedronToOFF(this.input.mesh, this.input.graph, outputFile, false, false);
			}
		}
		else {
			if(exportWithColors==true) {
				//MeshAlgorithms.writePolyhedronToOFF(this.input.mesh, outputFile, true, true);
			}
		}
	}*/

}
