package sd.viewer;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.Panel;
import java.awt.TextArea;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JToolBar;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import Jcg.mesh.MeshLoader;
import jdg.graph.AdjacencyListGraph;
import jdg.io.GraphReader;
import jdg.io.GraphReader_OFF;
import processing.core.PApplet;
import sd.SphericalLayoutComputation;
import sd.layout.Layout3D;
import sd.layout.SphericalLayout;

public class SphericalLayoutControlBar extends Panel implements ActionListener, ItemListener {

	private static final long serialVersionUID = 1L;
	private static final int buttonHeight=20;

	// GUI Widgets
	JButton buttonRandomPointsOnSphere, buttonRandomPointsInCube, buttonRandomPointsOnPlane; // compute random locations for graph vertices
	
	// sub-panels
	public PanelSphericalLayout panelSphericalLayout;
	public Panel3DEuclideanLayout panel3DEuclideanLayout;
	PanelViewer panelViewer;
	
	SphericalLayoutComputation layoutComputation; // main class (graph viewer)

	public SphericalLayoutControlBar(SphericalLayoutComputation layoutComputation, int width, int height) {
		this.layoutComputation=layoutComputation;
		this.setPreferredSize(new Dimension(width, height));
		this.setBackground(Color.lightGray);
		this.setLayout(new GridLayout(6, 2));
		//this.setLayout(new FlowLayout());
        this.setVisible(true);
		
		// add panels to the control bar
		this.panelViewer=new PanelViewer(this.layoutComputation.meshRendering, 60, width, 110);
		//this.panelViewer.buttonShow3DMesh.setEnabled(false);
		this.add(this.panelViewer);
		this.panelViewer.buttonShow3DMesh.setEnabled(false);
		this.panelViewer.buttonShowSphericalLayout.setEnabled(false);
		this.panelViewer.buttonGet3DMeshFromGraph.setEnabled(false);
		
		this.panelSphericalLayout=new PanelSphericalLayout(this.layoutComputation, 180, width, 230);
		this.add(this.panelSphericalLayout);

		this.panel3DEuclideanLayout=new Panel3DEuclideanLayout(this.layoutComputation, 420, width, 230);
		this.add(this.panel3DEuclideanLayout);
		
		//this.setPreferredSize(new Dimension(width, height));
		this.setBackground(Color.lightGray);
		//this.setLayout(new GridLayout(6, 2));
		this.setLayout(null);
        this.setVisible(true);
				
		this.buttonRandomPointsOnSphere = new JButton("random points on sphere");
		this.buttonRandomPointsOnSphere.setToolTipText("place vertices at random on the unit sphere");
		this.buttonRandomPointsOnSphere.addActionListener(this);
		this.buttonRandomPointsOnSphere.setBounds(0, 5, 240, buttonHeight);
		this.add(this.buttonRandomPointsOnSphere);
	
		this.buttonRandomPointsInCube = new JButton("random points in unit cube");
		this.buttonRandomPointsInCube.setToolTipText("place vertices at random in the unit cube");
		this.buttonRandomPointsInCube.addActionListener(this);
		this.buttonRandomPointsInCube.setBounds(0, 25, 240, buttonHeight);
		this.add(this.buttonRandomPointsInCube);
		
		this.repaint();	
	}
	
	public void actionPerformed(ActionEvent e) {		
		if (e.getSource() == this.buttonRandomPointsOnSphere) {
			SphericalLayout.setRandomPointsOnSphere(this.layoutComputation.meshRendering.input.graph);
		}

		if (e.getSource() == this.buttonRandomPointsInCube) {
			//SphericalLayout.setRandomPointsOnSphere(this.layoutComputation.meshRendering.input.graph);
			Layout3D.setRandomPoints3D(this.layoutComputation.input.graph, 1., 1., 1.);
		}

		this.repaint();
		this.layoutComputation.repaint();	// also repaint the canvas
	}
	
	@Override
	public void itemStateChanged(ItemEvent arg0) {
	}


}
