package sd.viewer;

import java.util.LinkedList;
import java.util.List;

import Jcg.geometry.Point_2;
import Jcg.geometry.Point_3;
import Jcg.polyhedron.Halfedge;
import Jcg.polyhedron.Polyhedron_3;
import Jcg.polyhedron.Vertex;
import Jcg.viewer.old.Fenetre;
import jdg.graph.AdjacencyListGraph;

public class Viewer2D {

	/**
	 * Draw a graph in a 2D frame <p>
	 * <p>
	 * Remark: the vertex coordinates are assumed to be stored in the global array 'coord2D'
	 */
	public static void draw2D(boolean[] isOnBoundary, boolean[] isInnerEdge, Point_2[] coord2D, AdjacencyListGraph graph, Polyhedron_3<Point_3> mesh) {
		System.out.println("Visualizing a graph layout in the plane");
		if(coord2D==null) {
			System.out.println("Warning: 2D coordinated not defined");
			return;
		}
		
		int n=graph.sizeVertices();
		
		Fenetre f=new Fenetre();
		
		List<Point_2[]> segments=new LinkedList<Point_2[]>(); // edge segments to draw
		
		for(Vertex u: mesh.vertices) {
				for(Halfedge e: (List<Halfedge>)u.getOutgoingHalfedges()) {
					if(e!=null) { // draw only directed edges (u, v) such that u<v, and draw only for indices in [0..n)
						int index1=e.getVertex().index;
						int index2=e.getOpposite().getVertex().index;
						
						if(index1<n && index2<n) {
							Point_2 p=coord2D[index1];
							Point_2 q=coord2D[index2];
							//if(isOnBoundary[u.index]==true && isOnBoundary[v.index]==true)
							//	f.addFatSegment(p, q);
							if(isInnerEdge[e.index]==true) {
								//f.addSegment(p, q);
								Point_2[] segment=new Point_2[] {p, q}; // edge segment (p, q) to draw
								//segments.add(segment);
								f.addSegment(segment[0], segment[1]);
							}
						}
					}
				}
		}
		
		//f.addColoredSegments(segments, null);
	}

	/**
	 * Export the list of edge segments of the Layour of the graph <p>
	 * <p>
	 * Remark: the vertex coordinates are assumed to be stored in the array 'coord2D'
	 */
	public static List<Point_2[]> exportEdgeList(boolean[] isOnBoundary, boolean[] isInnerEdge, Point_2[] coord2D, AdjacencyListGraph graph, Polyhedron_3<Point_3> mesh) {
		System.out.println("Exporting the list of edge segments (with geometric coordinates)");
		if(coord2D==null) {
			System.out.println("Warning: 2D coordinated not defined");
			return null;
		}
		
		int n=graph.sizeVertices();
		List<Point_2[]> segments=new LinkedList<Point_2[]>(); // edge segments to draw
		
		for(Vertex u: mesh.vertices) {
				for(Halfedge e: (List<Halfedge>)u.getOutgoingHalfedges()) {
					if(e!=null) { // draw only directed edges (u, v) such that u<v, and draw only for indices in [0..n)
						int index1=e.getVertex().index;
						int index2=e.getOpposite().getVertex().index;
						
						if(index1<n && index2<n) {
							Point_2 p=coord2D[index1];
							Point_2 q=coord2D[index2];
							//if(isOnBoundary[u.index]==true && isOnBoundary[v.index]==true)
							//	f.addFatSegment(p, q);
							if(isInnerEdge[e.index]==true) {
								//f.addSegment(p, q);
								Point_2[] segment=new Point_2[] {p, q}; // edge segment (p, q) to draw
								segments.add(segment);
							}
						}
					}
				}
		}
		
		return segments;
	}

	/**
	 * Draw a graph in a 2D rectangular frame (with positive 'x' and 'y' coordinates) <p>
	 * <p>
	 * Remark: the vertex coordinates, both 'x' and 'y' are assumed to be non negative
	 */
	public static void draw2D(double[] x, double[] y, AdjacencyListGraph graph, Polyhedron_3<Point_3> mesh) {
		System.out.println("Visualizing a rectangular layout in the plane");
		if(x==null || y==null) {
			System.out.println("Warning: 2D coordinated not defined");
			return;
		}
		
		int n=graph.sizeVertices();
		
		Fenetre f=new Fenetre();
		
		List<Point_2[]> segments=new LinkedList<Point_2[]>(); // edge segments to draw
		
		for(Vertex u: mesh.vertices) {
				for(Halfedge e: (List<Halfedge>)u.getOutgoingHalfedges()) {
					if(e!=null) { // draw only directed edges (u, v) such that u<v, and draw only for indices in [0..n)
						int index1=e.getVertex().index;
						int index2=e.getOpposite().getVertex().index;
						
						if(index1<n && index2<n) {
							Point_2 p=new Point_2(x[index1], y[index1]);
							Point_2 q=new Point_2(x[index2], x[index2]);
							if(x[e.index]>=0 && y[e.index]>=0) {
								//f.addSegment(p, q);
								Point_2[] segment=new Point_2[] {p, q}; // edge segment (p, q) to draw
								//segments.add(segment);
								f.addSegment(segment[0], segment[1]);
							}
						}
					}
				}
		}
		
		//f.addColoredSegments(segments, null);
	}

	/**
	 * Draw a graph in a 2D frame
	 * 
	 * @param points2D  2D coordinates of vertex locations
	 */
	public static void draw2D(boolean[] isInnerVertex, int[] x, int[] y, Vertex[] vertices, int k, AdjacencyListGraph graph, String title) {
		System.out.println("Visualizing a graph layout in the plane");
		
		int n=graph.sizeVertices();
		
		Fenetre f;
		if(title!=null)
			f=new Fenetre(title);
		else
			f=new Fenetre();
		
		List<Point_2[]> segments=new LinkedList<Point_2[]>(); // edge segments to draw
		
		for(int i=0;i<k;i++) {
			Vertex u=vertices[i];
			for(Halfedge e: (List<Halfedge>)u.getOutgoingHalfedges()) {
				if(e!=null) { // draw only directed edges (u, v) such that u<v, and draw only for indices in [0..n)
					int index1=e.getVertex().index;
					int index2=e.getOpposite().getVertex().index;
					
					if(index1<n && index2<n && isInnerVertex[index1]==true && isInnerVertex[index2]==true) {
						Point_2 p=new Point_2(x[index1], y[index1]);
						Point_2 q=new Point_2(x[index2], y[index2]);
						
						//f.addSegment(p, q);
						Point_2[] segment=new Point_2[] {p, q}; // edge segment (p, q) to draw
						//segments.add(segment);
						f.addSegment(segment[0], segment[1]);
					}
				}
			}
		}
		
		//f.addColoredSegments(segments, null);
	}
	
	/**
	 * Draw a graph in a 2D frame
	 * 
	 * @param points2D  2D coordinates of vertex locations
	 */
	public static List<Point_2[]> exportEdgeList(boolean[] isInnerVertex, int[] x, int[] y, Vertex[] vertices, int k, AdjacencyListGraph graph, String title) {
		System.out.println("Visualizing a graph layout in the plane");
		
		int n=graph.sizeVertices();
		
		List<Point_2[]> segments=new LinkedList<Point_2[]>(); // edge segments to draw
		
		for(int i=0;i<k;i++) {
			Vertex u=vertices[i];
			for(Halfedge e: (List<Halfedge>)u.getOutgoingHalfedges()) {
				if(e!=null) { // draw only directed edges (u, v) such that u<v, and draw only for indices in [0..n)
					int index1=e.getVertex().index;
					int index2=e.getOpposite().getVertex().index;
					
					if(index1<n && index2<n && isInnerVertex[index1]==true && isInnerVertex[index2]==true) {
						Point_2 p=new Point_2(x[index1], y[index1]);
						Point_2 q=new Point_2(x[index2], y[index2]);
						
						//f.addSegment(p, q);
						Point_2[] segment=new Point_2[] {p, q}; // edge segment (p, q) to draw
						segments.add(segment);
					}
				}
			}
		}
		
		return segments;
	}

}
